# This file is part of dds_example.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ["Component"]

import pathlib

import dds

HISTORY_TIMEOUT = 60  # max time for wait_for_historical_data (sec)


class Component:
    """Information for one DDS component, as described by one IDL file.

    Reads data from all topics for that component.

    Parameters
    ----------
    domain : `Domain`
        Domain instance.
    idl_path : `str`
        Path to IDL file that defines the topics for this component.
    partitions : `list` of `str`
        DDS partition names.
    """

    def __init__(self, domain, idl_path, partitions=("ddsexample",)):
        self.isopen = True
        self.domain = domain
        self.idl_path = pathlib.Path(idl_path).resolve()
        if not self.idl_path.is_file():
            raise ValueError(f"idl_path={idl_path} is not a file")
        self.name = self.idl_path.stem

        self.partitions = partitions

        # Create the publisher and subscriber. Both depend on the DDS
        # partitions, and so are created here instead of in Domain,
        # where most similar objects are created.
        partition_qos_policy = dds.PartitionQosPolicy(self.partitions)

        publisher_qos = domain.qos_provider.get_publisher_qos()
        publisher_qos.set_policies([partition_qos_policy])
        self.publisher = domain.participant.create_publisher(publisher_qos)

        subscriber_qos = domain.qos_provider.get_subscriber_qos()
        subscriber_qos.set_policies([partition_qos_policy])
        self.subscriber = domain.participant.create_subscriber(subscriber_qos)

    def close(self):
        """Shut down and clean up resources. A no-op if already closed."""
        if not self.isopen:
            return
        self.isopen = False
        self._guardcond.trigger()
        self._read_loop_task.cancel()
        while self.readers:
            read_cond, reader = self.readers.popitem()
            reader.close()
