# This file is part of dds_example.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ["Domain"]

import pathlib

import dds


class Domain:
    """Domain Participant and quality of service provider.

    You should only construct one of these per process.

    Parameters
    ----------
    qos_path : `str`
        Path to QoS XML file.
    """

    def __init__(self, qos_path):
        self.participant = None

        self.qos_path = pathlib.Path(qos_path).resolve()
        if not self.qos_path.is_file():
            raise ValueError(f"qos_path={qos_path} is not a file")
        self.qos_provider = dds.QosProvider(
            self.qos_path.as_uri(), "DDS DefaultQosProfile"
        )

        participant_qos = self.qos_provider.get_participant_qos()
        self.participant = dds.DomainParticipant(qos=participant_qos)

        self.topic_qos = self.qos_provider.get_topic_qos()
        self.reader_qos = self.qos_provider.get_reader_qos()
        self.writer_qos = self.qos_provider.get_writer_qos()

    @property
    def isopen(self):
        """Is the Domain open (usable)?"""
        return self.participant is not None

    def close(self):
        """Close all registered components and the dds domain participant.

        A no-op if already called."""
        if not self.isopen:
            return
        self.participant.close()
        self.participant = None

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()
