# This file is part of dds_example.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ["ReadTopic"]

import asyncio
import concurrent

import dds

from .base_topic import BaseTopic


class ReadTopic(BaseTopic):
    r"""Read a DDS topic.

    Accumulates read data in self.data

    Parameters
    ----------
    component : `Component`
        Component information
    name : `str`
        Topic name.

    Notes
    -----
    This data reads samples for one topic. The technique is as follows:

    * Run dds.WaitSet.wait in a background thread and return the
      read conditions with data to the asyncio caller.
      The caller then loops through the read conditions,
      looking up the associated reader in an internal dict
      (because unfortunately dds.ReadCondition does not appear to make the
      reader available) and deals with the data.

    As an alternative I tried runnning dds.WaitSet.wait in a loop
    in a thread and queuing the data from that loop. This sounds simple
    but getting data from a thread loop to an asyncio loop is tricky.
    I tried several variants of this, including one that used janus Queue
    (which is designed for this sort of thing) but none performed
    nearly as well as the current code. And the current code is simple
    and robust.
    """

    def __init__(self, *, component, name):
        super().__init__(component=component, name=name)
        self.isopen = True
        self.data = []
        self.reader = component.subscriber.create_datareader(
            self.topic, component.domain.reader_qos
        )
        self.read_condition = self.reader.create_readcondition(
            [
                dds.DDSStateKind.NOT_READ_SAMPLE_STATE,
                dds.DDSStateKind.ALIVE_INSTANCE_STATE,
            ]
        )
        self._wait_timeout = dds.DDSDuration(sec=10)
        self._guardcond = dds.GuardCondition()
        self._waitset = dds.WaitSet()
        self._waitset.attach(self.read_condition)
        self._waitset.attach(self._guardcond)
        self._read_loop_task = asyncio.ensure_future(self._read_loop())

    def close(self):
        """Shut down and release resources.
        """
        self.isopen = False
        self._guardcond.trigger()
        self._read_loop_task.cancel()
        self.reader.close()

    async def _read_loop(self):
        """Read and process data."""
        loop = asyncio.get_event_loop()
        try:
            with concurrent.futures.ThreadPoolExecutor(max_workers=1) as pool:
                while self.isopen:
                    await loop.run_in_executor(pool, self._wait_next)
                    if not self.isopen:
                        # Shutting down; stop reading.
                        return
                    # Odds are we will only get one value per read,
                    # but read everything, for efficiency.
                    data_list = self.reader.take_cond(self.read_condition, 100)
                    self.data += data_list
                    await asyncio.sleep(0)  # free the event loop
        except asyncio.CancelledError:
            raise
        except Exception as e:
            print("read loop failed: %s" % (e,))

    def _wait_next(self):
        """Wait for data to be available for any read topic.

        Blocks, so intended to be run in a background thread.

        Returns
        -------
        conditions : `List` of ``dds conditions``
            List of one or more dds read conditions and/or the guard condition
            which have been triggered.
        """
        conditions = self._waitset.wait(self._wait_timeout)
        return conditions
